"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeOrderCustomAttributes = exports.updateTagsForMultipleOrders = exports.updateTags = exports.markOrderAsPaid = exports.formatTax = exports.getOrder = exports.getOrders = void 0;
const utils_1 = require("./utils");
const api_1 = require("./api");
async function getOrders({ query, orderParams, limit = 250, }) {
    try {
        const res1 = await api_1.graphqlClient.request(`{
            orders(first: ${limit},query: "${query}") {
              edges {
                node {
                  id
                }
              }
            }
          }`);
        const orderQueue = res1.orders.edges.map(({ node }) => ({
            orderId: node.id,
            orderParams,
        }));
        const res2 = await (0, utils_1.throttle)(getOrder, orderQueue, 4, 1500);
        const successfulRes = (0, utils_1.getThrottleResValues)(res2);
        return successfulRes;
    }
    catch (e) {
        throw e;
    }
}
exports.getOrders = getOrders;
async function getOrder({ orderId, orderParams, onlyTags = false, }) {
    const res = await api_1.graphqlClient.request(`{
              node(id: "${orderId}") {
                id
                ...on Order {
                  ${onlyTags ? "tags" : orderParams}
                }
              }
            }`);
    return res.node;
}
exports.getOrder = getOrder;
function formatTax(orderOrOrders) {
    const format = (node) => {
        node.lineItems.edges.forEach(({ node: item }) => {
            const data = {
                priceSet: 0,
                ratePercentage: 0,
            };
            item.taxLines.forEach(({ priceSet: { presentmentMoney: { amount: priceSet }, }, ratePercentage, }) => {
                data.priceSet += parseFloat(priceSet);
                data.ratePercentage += ratePercentage;
            });
            item.taxLine = data;
        });
        return node;
    };
    if (Array.isArray(orderOrOrders)) {
        orderOrOrders.forEach((order) => {
            format(order);
        });
    }
    else {
        format(orderOrOrders);
    }
}
exports.formatTax = formatTax;
async function markOrderAsPaid(orderId) {
    try {
        const res = await api_1.graphqlClient.request(`mutation orderMarkAsPaid($input: OrderMarkAsPaidInput!) {
                orderMarkAsPaid(input: $input) {
                  order {
                    id
                  }
                  userErrors {
                    field
                    message
                  }
                }
              }`, { input: { id: orderId } });
        return res.orderMarkAsPaid.order;
    }
    catch (e) {
        throw e;
    }
}
exports.markOrderAsPaid = markOrderAsPaid;
async function updateTags(id, ...newTags) {
    const order = await getOrder({
        orderId: id,
        orderParams: `tags`,
        onlyTags: true,
    });
    const existingTags = order.tags;
    const finalTags = [...new Set([...existingTags, ...newTags])];
    const res = await api_1.graphqlClient.request(`mutation orderUpdate($input: OrderInput!) {
                orderUpdate(input: $input) {
                  userErrors {
                    field
                    message
                  }
                  order {
                    name
                  }
                }
              }`, { input: { id, tags: finalTags } });
    const returnVal = { orderId: id, tagged: false };
    if (res.orderUpdate.order)
        returnVal.tagged = true;
    return returnVal;
}
exports.updateTags = updateTags;
async function updateTagsForMultipleOrders(orderIds, ...newTags) {
    const req = async (orderId) => {
        return await updateTags(orderId, ...newTags);
    };
    const res = await (0, utils_1.throttle)(req, orderIds, 1, 0);
    return (0, utils_1.getThrottleResValues)(res);
}
exports.updateTagsForMultipleOrders = updateTagsForMultipleOrders;
function optimizeOrderCustomAttributes(customAttributes) {
    const map = new Map();
    customAttributes.forEach(({ key, value }) => {
        map.set(key, value);
    });
    return map;
}
exports.optimizeOrderCustomAttributes = optimizeOrderCustomAttributes;
